"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MarkdownParser = void 0;
const tokenize_1 = require("./tokenize");
const parser_1 = require("./parser");
const plugins_1 = require("./plugins");
/**
 * Markdown parser class
 */
class MarkdownParser {
    constructor() {
        this.buffer = "";
        // Initialize with default plugins (excluding syntaxHighlightPlugin)
        this.plugins = [
            plugins_1.tablePlugin.bind(this),
            plugins_1.inlineStylesPlugin.bind(this),
            plugins_1.imagePlugin.bind(this),
            plugins_1.linkPlugin.bind(this),
        ];
        // Syntax highlighting is disabled by default
        this.useSyntaxHighlighting = false;
        this.inCodeBlock = false;
        this.currentList = null;
        this.reset();
    }
    /**
     * Allows additional plugins to be registered
     * @param plugin A function that processes a token and returns HTML or null
     */
    use(plugin) {
        this.plugins.push(plugin);
        return this;
    }
    /**
     * Enables or disables syntax highlighting
     * @param enable A boolean indicating whether to enable syntax highlighting
     */
    enableSyntaxHighlighting(enable) {
        this.useSyntaxHighlighting = enable;
        if (enable) {
            // Add syntaxHighlightPlugin if not already present
            if (!this.plugins.includes(plugins_1.syntaxHighlightPlugin)) {
                this.plugins.push(plugins_1.syntaxHighlightPlugin.bind(this));
            }
        }
        else {
            // Remove syntaxHighlightPlugin if it exists
            this.plugins = this.plugins.filter((plugin) => plugin !== plugins_1.syntaxHighlightPlugin);
        }
        return this;
    }
    /**
     * Renders Markdown input into HTML
     * @param input The Markdown string to be rendered
     * @returns Rendered HTML string
     */
    render(input) {
        // Tokenize the input
        const tokens = (0, tokenize_1.tokenize)(input);
        // Parse the tokens into HTML
        return (0, parser_1.parse)(tokens, this.plugins);
    }
    /**
     * Resets the parser to its initial state
     */
    // New methods for streaming support
    reset() {
        // Initialize all stateful properties
        this.inCodeBlock = false;
        this.currentList = null;
        this.buffer = "";
        // Add other stateful properties from your original implementation
    }
    /**
     * Parses a chunk of Markdown text
     * @param chunk A chunk of Markdown text
     * @returns The parsed HTML string
     */
    // New streaming-friendly parse method
    parseChunk(chunk) {
        this.buffer += chunk;
        const lines = this.buffer.split("\n");
        this.buffer = lines.pop() || "";
        let html = "";
        for (const line of lines) {
            html += this._parseLine(line);
        }
        return html;
    }
    finalize() {
        let html = "";
        if (this.buffer) {
            html += this._parseLine(this.buffer);
        }
        // Close any remaining open blocks
        if (this.inCodeBlock)
            html += "</code></pre>";
        this.reset();
        return html;
    }
    tokenizeStream(line) {
        var _a;
        // Implement your actual tokenization logic here
        const tokens = [];
        // Example simple tokenization:
        if (line.startsWith("```")) {
            tokens.push({
                type: "code_block",
                content: line.trim(),
                language: line.replace(/```/g, "").trim(),
            });
        }
        else if (line.startsWith("#")) {
            const level = ((_a = line.match(/#/g)) === null || _a === void 0 ? void 0 : _a.length) || 1;
            tokens.push({
                type: "heading",
                content: line.replace(/#/g, "").trim(),
                level: Math.min(level, 6),
            });
        }
        else {
            tokens.push({ type: "paragraph", content: line });
        }
        return tokens;
    }
    // Private method for line parsing
    _parseLine(line) {
        const tokens = this.tokenizeStream(line);
        // console.log("token-->", tokens);
        // Your existing parsing logic adapted for single lines
        // ... modified to use this.inCodeBlock, this.currentList etc ...
        let lineHtml = ""; // Assuming lineHtml is a string
        if (this.inCodeBlock) {
            lineHtml += this.parseCodeBlockLine(tokens);
        }
        else {
            lineHtml += this.parseRegularLine(tokens);
        }
        return lineHtml;
    }
    parseRegularLine(tokens) {
        let html = "";
        for (const token of tokens) {
            switch (token.type) {
                case "heading":
                    html += `<h${token.level}>${token.content}</h${token.level}>`;
                    break;
                case "code_block":
                    this.inCodeBlock = true;
                    html += `<pre><code class="language-${token.language}">`;
                    break;
                case "paragraph":
                    html += this.parseInlineElements(token.content || "");
                    break;
            }
        }
        return html;
    }
    parseInlineElements(text) {
        // Simple inline parsing example
        return `<p>${text
            .replace(/\*\*(.*?)\*\*/g, "<strong>$1</strong>")
            .replace(/\*(.*?)\*/g, "<em>$1</em>")}</p>`;
    }
    parseCodeBlockLine(tokens) {
        let html = "";
        for (const token of tokens) {
            if (token.type === "code_block") {
                this.inCodeBlock = false;
                return "</code></pre>";
            }
            html += `${token.content}\n`;
        }
        return html;
    }
}
exports.MarkdownParser = MarkdownParser;
